package com.alicloud.openservices.tablestore.core.protocol;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.alicloud.openservices.tablestore.ClientException;
import com.alicloud.openservices.tablestore.core.ResponseContentWithMeta;
import com.alicloud.openservices.tablestore.model.*;
import java.nio.ByteBuffer;
import java.util.Formatter;

public class ResponseFactory {

    public static CreateTableResponse createCreateTableResponse(
        ResponseContentWithMeta response,
        OtsInternalApi.CreateTableResponse createTableResponse)
    {
        return new CreateTableResponse(response.getMeta());
    }

    public static ListTableResponse createListTableResponse(
        ResponseContentWithMeta response,
        OtsInternalApi.ListTableResponse listTableResponse)
    {
        ListTableResponse result = new ListTableResponse(response.getMeta());
        result.setTableNames(listTableResponse.getTableNamesList());
        return result;
    }

    public static DescribeTableResponse createDescribeTableResponse(
        ResponseContentWithMeta response,
        OtsInternalApi.DescribeTableResponse describeTableResponse)
    {
        try {
            DescribeTableResponse result = new DescribeTableResponse(response.getMeta());
            result.setTableMeta(OTSProtocolParser.parseTableMeta(describeTableResponse.getTableMeta()));
            result.setReservedThroughputDetails(OTSProtocolParser.parseReservedThroughputDetails(describeTableResponse.getReservedThroughputDetails()));
            result.setTableOptions(OTSProtocolParser.parseTableOptions(describeTableResponse.getTableOptions()));
            List<PrimaryKey> shards = new ArrayList<PrimaryKey>();
            for(int i = 0; i < describeTableResponse.getShardSplitsCount(); ++i) {
                com.google.protobuf.ByteString bytes = describeTableResponse.getShardSplits(i);
                PlainBufferCodedInputStream coded = new PlainBufferCodedInputStream(new PlainBufferInputStream(bytes.asReadOnlyByteBuffer()));
                List<PlainBufferRow> rows = coded.readRowsWithHeader();
                if (rows.size() != 1) {
                    throw new IOException("Expect only returns one row. Row count: " + rows.size());
                }
                Row row = PlainBufferConversion.toRow(rows.get(0));
                shards.add(row.getPrimaryKey());
            }
            result.setShardSplits(shards);
            return result;
        } catch (Exception e) {
            throw new ClientException("Failed to parse describe table response.", e);
        }
    }

    public static DeleteTableResponse createDeleteTableResponse(
        ResponseContentWithMeta response,
        OtsInternalApi.DeleteTableResponse deleteTableResponse)
    {
        return new DeleteTableResponse(response.getMeta());
    }

    public static UpdateTableResponse createUpdateTableResponse(
        ResponseContentWithMeta response,
        OtsInternalApi.UpdateTableResponse updateTableResponse)
    {
        UpdateTableResponse result = new UpdateTableResponse(response.getMeta());
        result.setReservedThroughputDetails(OTSProtocolParser.parseReservedThroughputDetails(updateTableResponse.getReservedThroughputDetails()));
        result.setTableOptions(OTSProtocolParser.parseTableOptions(updateTableResponse.getTableOptions()));
        return result;
    }

    public static GetRowResponse createGetRowResponse(
            ResponseContentWithMeta response, OtsInternalApi.GetRowResponse getRowResponse) {
        ConsumedCapacity consumedCapacity = new ConsumedCapacity(OTSProtocolParser.parseCapacityUnit(getRowResponse.getConsumed().getCapacityUnit()));
        Row row = null;
        if (!getRowResponse.getRow().isEmpty()) {
            try {
                PlainBufferCodedInputStream inputStream = new PlainBufferCodedInputStream(new PlainBufferInputStream(getRowResponse.getRow().asReadOnlyByteBuffer()));
                List<PlainBufferRow> rows = inputStream.readRowsWithHeader();
                if (rows.size() != 1) {
                    throw new IOException("Expect only returns one row. Row count: " + rows.size());
                }
                row = PlainBufferConversion.toRow(rows.get(0));
            } catch (Exception e) {
                throw new ClientException("Failed to parse row", e);
            }
        }
        GetRowResponse result = new GetRowResponse(response.getMeta(), row, consumedCapacity);
        if (getRowResponse.hasNextToken()) {
            result.setNextToken(getRowResponse.getNextToken().toByteArray());
        }
        return result;
    }

    public static PutRowResponse createPutRowResponse(
            ResponseContentWithMeta response, OtsInternalApi.PutRowResponse putRowResponse) {
        ConsumedCapacity consumedCapacity = new ConsumedCapacity(OTSProtocolParser.parseCapacityUnit(putRowResponse.getConsumed().getCapacityUnit()));
        PutRowResponse result = new PutRowResponse(response.getMeta(), consumedCapacity);
        return result;
    }

    public static UpdateRowResponse createUpdateRowResponse(
            ResponseContentWithMeta response, OtsInternalApi.UpdateRowResponse updateRowResponse) {
        ConsumedCapacity consumedCapacity = new ConsumedCapacity(OTSProtocolParser.parseCapacityUnit(updateRowResponse.getConsumed().getCapacityUnit()));
        UpdateRowResponse result = new UpdateRowResponse(response.getMeta(), consumedCapacity);
        return result;
    }

    public static DeleteRowResponse createDeleteRowResponse(
            ResponseContentWithMeta response, OtsInternalApi.DeleteRowResponse deleteRowResponse) {
        ConsumedCapacity consumedCapacity = new ConsumedCapacity(OTSProtocolParser.parseCapacityUnit(deleteRowResponse.getConsumed().getCapacityUnit()));
        DeleteRowResponse result = new DeleteRowResponse(response.getMeta(), consumedCapacity);
        return result;
    }

    public static GetRangeResponse createGetRangeResponse(
            ResponseContentWithMeta response, OtsInternalApi.GetRangeResponse getRangeResponse) {
        try {
            ConsumedCapacity consumedCapacity = new ConsumedCapacity(OTSProtocolParser.parseCapacityUnit(getRangeResponse.getConsumed().getCapacityUnit()));
        	GetRangeResponse result = new GetRangeResponse(response.getMeta(), consumedCapacity);

            if (!getRangeResponse.hasNextStartPrimaryKey()) {
                // has no next primary key
                result.setNextStartPrimaryKey(null);
            } else {
                PlainBufferCodedInputStream inputStream = new PlainBufferCodedInputStream(new PlainBufferInputStream(getRangeResponse.getNextStartPrimaryKey().asReadOnlyByteBuffer()));
                List<PlainBufferRow> rows = inputStream.readRowsWithHeader();
                if (rows.size() != 1) {
                    throw new IOException("Expect only returns one row. Row count: " + rows.size());
                }
                PlainBufferRow row = rows.get(0);
                if (row.hasDeleteMarker() || row.hasCells()) {
                    throw new IOException("The next primary key should only have primary key: " + row);
                }

                result.setNextStartPrimaryKey(PlainBufferConversion.toPrimaryKey(row.getPrimaryKey()));
            }

            if (!getRangeResponse.getRows().isEmpty()) {
                List<Row> rows = new ArrayList<Row>();
                PlainBufferCodedInputStream inputStream = new PlainBufferCodedInputStream(new PlainBufferInputStream(getRangeResponse.getRows().asReadOnlyByteBuffer()));
                List<PlainBufferRow> pbRows = inputStream.readRowsWithHeader();
                for (PlainBufferRow pbRow : pbRows) {
                    rows.add(PlainBufferConversion.toRow(pbRow));
                }
                result.setRows(rows);
            } else {
                result.setRows(new ArrayList<Row>());
            }

            if (getRangeResponse.hasNextToken()) {
                result.setNextToken(getRangeResponse.getNextToken().toByteArray());
            }

            return result;
        } catch (Exception e) {
            throw new ClientException("Failed to parse get range response.", e);
        }
    }

    public static BatchGetRowResponse createBatchGetRowResponse(
            ResponseContentWithMeta response,
            OtsInternalApi.BatchGetRowResponse batchGetRowResponse) {
    	BatchGetRowResponse result = new BatchGetRowResponse(response.getMeta());

        for (OtsInternalApi.TableInBatchGetRowResponse table : batchGetRowResponse.getTablesList()) {
            String tableName = table.getTableName();
            List<OtsInternalApi.RowInBatchGetRowResponse> rowList = table.getRowsList();
            for (int i = 0; i < rowList.size(); i++) {
                result.addResult(OTSProtocolParser.parseBatchGetRowStatus(tableName, rowList.get(i), i));
            }
        }
        return result;
    }

    public static BatchWriteRowResponse createBatchWriteRowResponse(
            ResponseContentWithMeta response,
            OtsInternalApi.BatchWriteRowResponse batchWriteRowResponse) {
    	BatchWriteRowResponse result = new BatchWriteRowResponse(response.getMeta());

        for (OtsInternalApi.TableInBatchWriteRowResponse table : batchWriteRowResponse.getTablesList()) {
            String tableName = table.getTableName();

            List<OtsInternalApi.RowInBatchWriteRowResponse> statuses = table.getRowsList();
            for (int i = 0; i < statuses.size(); i++) {
                result.addRowResult(OTSProtocolParser.parseBatchWriteRowStatus(tableName, statuses.get(i), i));
            }
        }

        return result;
    }
}
